# Copyright(c) 2004-2006 Association of Universities for Research in Astronony, Inc.
#
###############################################################################
# Gemini GNIRS data reduction script                                          #
# Reduction for: longslit calibrations                                        #
#                                                                             #
# This script is provided to guide the GNIRS data user through the reduction  #
# process.  It is not intended to be an executable task, or a "plug-n-play"   #
# batch job.  It does show the reduction steps using actual GNIRS data and    #
# provides explanatory comments at each step.                                 #
#                                                                             #
# At a minimum, the user needs to edit this file to match his/her dataset.    #
# It is also strongly recommended that the user read the embedded comments    #
# and understand the processing steps, as the optimum steps for a given       #
# dataset may differ.                                                         #
#                                                                             #
# Once properly edited, the script can be run by cutting and pasting the      #
# text into an IRAF session, or by typing                                     #
#   gn> task $thisscript="thisscript.cl"                                      #
#   gn> thisscript                                                            #
# in the IRAF session                                                         #
#                                                                             #
# WARNING: Certain INTERACTIVE steps in the process MAY NOT WORK WELL         #
#          with the 'cl < thisscriptname.cl' syntax).  Likewise, when         #
#          cutting/pasting, DO NOT paste commands directly after an           #
#          interactive task in one paste (that is paste the following         #
#          command after the interactive task).                               #
#                                                                             #
# Note that this script is designed to be re-run as needed, and so each step  #
# is preceeded by a command to delete the files created in that step.         #
###############################################################################
#
# LS32 Calibration Dataset:
#
# Observation UT date : 2004 Jun 15
# Data filename prefix: S20040615S 
#
# File numbers:
#       IR lamp flats:  184-194
#       Argon arcs:     195-196
#       Telluric:       145-152
#
#  This is an example of a basic calibration reduction for longslit/32 
#   (R~1700) dataset with central wavelength=2.2microns.  The data consist 
#   of GCAL flats, an arc and a telluric standard.  
#
###############################################################################
#
#  This script can also be used as a guide for reduction of longslit/111
#   (R~6000) data.  The reduction steps are identical; what differences 
#   there are are in parameter settings and these are identified in comments
#   starting with "LS111:".
#
#  LS111 Calibration Dataset:
#
# Observation UT date : 2004 Jun 13
# Data filename prefix: S20040613S
#
# File numbers:
#       IR lamp flats:  171-180
#       Argon arcs:     182
#       Telluric:       161-168
#       Pinhole:        183
#
# To run the LS111 case, modify the gemlist commands below for these datasets.
#
###############################################################################
# Last revised: July 2006, Thebaut Jonathan and Bernadette Rodgers
#


######################################################################
		### Initialize tasks ###
######################################################################
# Load required tasks/packages 
gemini
gnirs

#-----------setup for gnirs------------
# set default parameters
# use gnirs defaults except where specified on command lines below
unlearn gemini
unlearn gemtools
unlearn gnirs


######################################################################
		### define logfile and database ###
######################################################################

# set logfile
# To start from scratch, delete existing logfile
#   delete gnirs_ls-cal.log ver- >& dev$null
gnirs.logfile="gnirs_ls-cal.log"

#set database dir
gnirs.database="gnirs_ls_database"

# to start from scratch, delete database entries if present
#   delete gnirs_ls-cal_database/* ver- >& dev$null

# load gnirs header keywords
nsheaders gnirs

# set display
set stdimage=imt1024


######################################################################
		### Create lists and path to data ###
######################################################################

# This line points to the raw data 
set rawdir="/net/sabrina/staging5/gemini_testdata/gnirs_testdata/longslit/"

#-----Create lists-----
delete flat.lis ver- >& dev$null
gemlist S20040615S 184-194 > flat.lis

delete arc.lis ver- >& dev$null
gemlist S20040615S 195 > arc.lis

delete telluric.lis ver- >& dev$null
gemlist S20040615S 145-152 > telluric.lis

delete all.lis ver- >& dev$null
cat flat.lis arc.lis telluric.lis >> all.lis

######################################################################
		### nsprepare the data ###
######################################################################

# Examine all data visually first.  If data show a strong vertical striping
# pattern (due to variable bias problem on readout), run nvnoise on the raw
# data to correct this.  This task subtracts a constant from each pixel
# and so does not add any noise.
# 
## If nvnoise is run, all subsequent files will have a 'v' preceeding the 
## file prefix (by default).  This is not included in the subsequent steps
## in this example.
#
# imdelete v@all.lis ver- >& dev$null
# nvnoise @all.lis
#

# run nsprepare to correct for non-linearity, determine MDF shifts,
# fix headers and create variance and data quality planes.  If a flat
# is included in the file list and shiftx and shifty = INDEF, nsprepare
# will determine the MDF shift from the flat and insert in all file headers.
# If running nsprepare on a dataset that does not include a
# flat, use the "shiftimage" parameter to match the shift to an 
# nsprepare-d flat image.

# Use fl_forcewcs=yes to correct the WCS keywords in the headers.
# Bad pixel mask added to GNIRS IRAF package with version 1.9.

imdelete n@all.lis ver- >& dev$null
nsprepare @all.lis rawpath=rawdir$ shiftx=INDEF shifty=INDEF fl_force+ \
    bpm="gnirs$data/gnirs_2005sep24_bpm.fits"



######################################################################
		### make flat ###
######################################################################

## print imstatistics on flats to check for anomalous mean or
## standard deviation.  Often the first flat in a set should be discarded.
## Note: these steps create and delete a file called "tmp.lis"
## Prints to screen and to log file

delete tmp.lis ver- >& dev$null
printlog ("--------------------------------------------", gnirs.logfile, ver+)
gemextn n@flat.lis proc="expand" extname="SCI" > tmp.lis
printlog ("Flat Statistics: ", gnirs.logfile, verbose+)
imstat @tmp.lis | tee (gnirs.logfile, append+)
printlog ("--------------------------------------------", gnirs.logfile, ver+)
delete tmp.lis ver- >& dev$null

# the first two are slightly off from the rest of the set (MEAN value)
# better to build the flat without these frames

delete flat.lis ver- >& dev$null
gemlist S20040615S 186-194 > flat.lis
imdelete final_flat,final_flat_bpm.pl ver- >& dev$null

# not required to cut longslit data, but it avoids a large number of 
# "bad" pixels from the unilluminated edges in the DQ array

imdelete rn@flat.lis ver- >& dev$null
nsreduce n@flat.lis fl_nscut+ fl_nsappwav- fl_sky- fl_flat- 

# no darks are needed for such short exposure times (only add noise)
nsflat rn@flat.lis darks="" flatfile="final_flat.fits"
display final_flat[sci,1] 1



######################################################################
		### wavelength calibrate arc ###
######################################################################

imdelete rn@arc.lis ver- >& dev$null
nsreduce n@arc.lis fl_sky- fl_flat- 
imdelete wrn@arc.lis ver- >& dev$null

# If re-running, be sure to delete previous results!
#   delete gnirs_ls_database/idwrnS20040615S0195_SCI_1_  ver- >& dev$null
# For lower dispersion, this task often works well non-interactively, BUT 
# ALWAYS CHECK RESULT-- you might need to do it interactively.

# NOTE: Commands with fl_inter+ should be pasted onto the IRAF command line
#      (they may not work properly when run in 'cl < script.cl' batch mode)

nswavelength rn@arc.lis 

## For LS111: 
##   Need to increase cradius for 111 resolution (cradius=20)
##   If there are very few lines in your arc, reduce order (default=4)
##   Default threshold=100, may want to reduce for very faint lines (but run
##   interactively to be sure not to identify on noise!)
#
#nswavelength rn@arc.lis cradius=20. threshold=50. order=2 fl_inter+


######################################################################
		### reduce tellurics ###
######################################################################

## There are 3 ways to determine the correct frames for sky subtraction
## in nsreduce.  If the frames are evenly spaced in time (i.e., ABBA 
## dither pattern), the easiest is "skyrange=INDEF" (the default), which
## will determine the best sky frame for each image from the input list.
## One can also set the skyrange (in seconds) by hand, or provide a list
## of sky images directly.  See nsreduce help for more information.
## If two or more images meet the sky selection criteria (skyrange and 
## nodsize), they will be combined to create the sky frame that is subtracted.

## The following steps can be used to determine the spacing in time between
## images, if needed to set skyrange.

delete tmp.lis >& dev$null
printlog ("--------------------------------------------", gnirs.logfile, ver+)
gemextn n@telluric.lis proc="expand" index="0" > tmp.lis
printlog ("Telluric Exposure Times: ", gnirs.logfile, ver+)
hselect @tmp.lis "$I,UT" yes | tee (gnirs.logfile, append+)
printlog ("--------------------------------------------", gnirs.logfile, ver+)
delete tmp.lis ver- >& dev$null

# ~20 seconds will include neighbours for sky, use this value for the 
# skyrange parameter.

imdelete rn@telluric.lis ver- >& dev$null
nsreduce n@telluric.lis fl_nsapp- skyrange=20 fl_flat+ flatim=final_flat

## For LS111 data:
#nsreduce n@telluric.lis fl_nsapp- skyrange=25. fl_flat+ flatim=final_flat 


##################
# FYI:
#
# NSSSTACK combines things at the same offset position (no shifting) 
#   and gives 1 output file per position.
# NSCOMBINE (used below) shifts all data spatially to a common offset 
#   position and gives 1 combined output file.
################## 


## NSTRANSFORM
# call nstransform to straighten and dispersion correct, then NSCOMBINE 
# on reduced frames 

# Here NSTRANSFORM is doing the dispersion correction only.  Spatial
# rectification (s-distortion correction) is not usually needed with 
# GNIRS long slit data.  If it is desired, one must first call NSSDIST
# with a pinhole exposure (see XD-CAL example).

imdelete trn@telluric.lis ver- >& dev$null
nstransform rn@telluric.lis lamp=wrnS20040615S0195  

# combine 2D spectra 
imdelete trnS20040615S0145_comb ver- >& dev$null
nscombine trn@telluric.lis

# now extract to 1-D
imdelete xtrnS20040615S0145_comb ver- >& dev$null
nsextract trnS20040615S0145_comb

#final s/n ~50
splot xtrnS20040615S0145_comb[sci,1]



######################### FINISHED  ##########################################  

# Cleanup: 
#delete tmp*,*.lis ver- >& dev$null
#imdelete nS20040615S????.fits ver- >& dev$null
#imdelete snS20040615S????.fits ver- >& dev$null
#imdelete rnS20040615S????.fits ver- >& dev$null
#imdelete trnS20040615S????.fits ver- >& dev$null
