# Copyright(c) 2004-2006 Association of Universities for Research in Astronony, Inc.

###############################################################################
# Gemini GNIRS data reduction script                                          #
# Reduction for: longslit Science                                             #
#                                                                             #
# This script is provided to guide the GNIRS data user through the reduction  #
# process.  It is not intended to be an executable task, or a "plug-n-play"   #
# batch job.  It does show the reduction steps using actual GNIRS data and    #
# provides explanatory comments at each step.                                 #
#                                                                             #
# At a minimum, the user needs to edit this file to match his/her dataset.    #
# It is also strongly recommended that the user read the embedded comments    #
# and understand the processing steps, as the optimum steps for a given       #
# dataset may differ.                                                         #
#                                                                             #
# Once properly edited, the script can be run by cutting and pasting the      #
# text into an IRAF session, or by typing                                     #
#   gn> task $thisscript="thisscript.cl"                                      #
#   gn> thisscript                                                            #
# in the IRAF session                                                         #
#                                                                             #
# WARNING: Certain INTERACTIVE steps in the process MAY NOT WORK WELL         #
#          with the 'cl < thisscriptname.cl' syntax).  Likewise, when         #
#          cutting/pasting, DO NOT paste commands directly after an           #
#          interactive task in one paste (that is paste the following         #
#          command after the interactive task).                               #
#                                                                             #
# Note that this script is designed to be re-run as needed, and so each step  #
# is preceeded by a command to delete the files created in that step.         #
###############################################################################
#
# LS32 Science Dataset:
#
# Observation UT date : 2004 Jun 15
# Data filename prefix: S20040615S
#
# Science target file numbers: 156-183  (GS-2004A-SV-8)
#
## This is an example of basic data reduction for longslit moderate (R~1700) 
# resolution data of an extended source.  This case is different from the other 
# examples in that the data were taken with an obj-sky dither pattern, i.e., 
# only 50% of data are taken on-source (as opposed to dithering along the slit).
#
# This reduction assumes the presence of the following files in the caldir
# directory:
# 	Flat :  final_flat.fits 
# 	Arc  :  wrnS20040615S0195.fits 
#	Telluric : xtrnS20040615S0145_comb.fits
#
#  These files can be created with the LS-Cal example.
#
# The data used in this script are available in the Gemini Science Archive. 
#
###############################################################################
#
#  This script can also be used as a guide for reduction of longslit/111 (R~6000)
#  science data.  The reduction steps are identical; differences in parameter
#  settings are indicated in the comments starting with "LS111:"
#
# LS111 Science Dataset:
#
# Observation UT date : 2004 Jun 13
# Data filename prefix: S20040613S
#
# Science target file numbers: 124-155  (GS-2004A-SV-15)
#
## This is an example of basic data reduction for longslit moderate (R~1700) 
# resolution data of an extended source.  This case is different from the other 
# examples in that the data were taken with an obj-sky dither pattern, i.e., 
# only 50% of data are taken on-source (as opposed to dithering along the slit).
#
# This reduction assumes the presence of the following files in the caldir
# directory:
# 	Flat :  final_flat.fits 
# 	Arc  :  wrnS20040613S0182.fits
#       Telluric: xtrnS20040613S0161_comb.fits 
#
#  These files can be created with the LS-Cal example, modified for 111 dataset.
#
# The data used in this script are available in the Gemini Science Archive. 
#
###############################################################################
# Last revised: Jul 2006, Thebaut Jonathan & Bernadette Rodgers
#


######################################################################
		### Initialize tasks ###
######################################################################

#Load required tasks/packages
gemini
gnirs

#-----------setup for gnirs----------
#use gnirs defaults except where specified on command lines below
unlearn gemini
unlearn gemtools
unlearn gnirs


######################################################################
		### define logfile and database ###
######################################################################


# This line points to the raw data 
set rawdir=/net/sabrina/staging5/gemini_testdata/gnirs_testdata/longslit/

# If the calibration were reduced in a different directory, define 'caldir'
#set caldir=rawdir$LS32example-CAL/

# set logfile
delete gnirs_ls-sci.log ver- >& dev$null
gnirs.logfile="gnirs_ls-sci.log"

#set database dir
#gnirs.database="gnirs_ls_database"

# load header keywords
nsheaders gnirs

# set display
set stdimage=imt1024



######################################################################
		### Create lists and path to data ###
######################################################################


#------create lists---------
# need separate files for object and sky
delete obj.lis,sky.lis ver- >& dev$null
gemlist S20040615S 156,159,160,163,164,167,168,171,172,175,176,179,180,183 \
    > obj.lis
gemlist S20040615S 157,158,161,162,165,166,169,170,173,174,177,178,181,182 \
    > sky.lis



######################################################################
		### nsprepare the data ###
######################################################################

# shift taken from calibration example, where measured with cross-correlation
imdelete n@obj.lis,n@sky.lis ver- >& dev$null
nsprepare @obj.lis,@sky.lis rawpath=rawdir$ shiftx=3.3


######################################################################
		### Objects ###
######################################################################

### - Reduce Objects ###

# reduce object frames; all images in the supplied sky list will be combined 
# and subtracted from all input frames.

delete phu.lis ver- >& dev$null
printlog ("--------------------------------------------", gnirs.logfile, ver+)
gemextn n@obj.lis proc="expand" index="0" > phu.lis
printlog ("Exposure Time Stamps for skyrange parameter", gnirs.logfile, ver+)
hselect @phu.lis "$I,UT" yes | tee (gnirs.logfile, append+)
printlog ("--------------------------------------------", gnirs.logfile, ver+)

imdelete rn@obj.lis ver- >& dev$null
nsreduce n@obj.lis fl_nscut+ fl_dark- fl_sky+ skyimages=n@sky.lis \
  fl_flat+ flatim=final_flat.fits skyrange=180

# If the calibrations were reduced in a different directory:
#nsreduce n@obj.lis fl_nscut+ fl_dark- fl_sky+ skyimages=n@sky.lis \
#   fl_flat+ flatim=caldir$final_flat.fits skyrange=180

# all source frames are at the same position, no need to use nscomb
imdelete rnS20040615S0156_stack.fits ver- >& dev$null
nsstack rn@obj.lis


### - Transform Objects ###

# now transform final 2-D image
#   - NSTRANSFORM can apply both wavelength and s-distortion transformations.
#   - need local copy of fields used for calibration

# If the calibrations were reduced in a different directory:
#imdelete wrnS20040615S0195 ver- >& dev$null
#copy caldir$wrnS20040615S0195.fits .
#if (access ("gnirs_ls_database"))
#    delete gnirs_ls_database/* ver- >& dev$null
#else
#    !mkdir gnirs_ls_database
#copy caldir$gnirs_ls_database/* gnirs_ls_database/

# run nstransform
imdelete trnS20040615S0156_stack.fits ver- >& dev$null
nstransform rnS20040615S0156_stack.fits lamp=wrnS20040615S0195 \
    database=gnirs_ls_database/ 


### - extract data, if desired ###

imdelete xtrnS20040615S0156_stack.fits ver- >& dev$null
nsextract trnS20040615S0156_stack.fits

# s/n ~6
splot xtrnS20040615S0156_stack[sci,1]



### - telluric correction  ###

## Note: this telluric has not been corrected to remove intrinsic
##       stellar features-- this will leave a false emission feature in
##       the final spectrum at the position of the telluric Br-gamma line.

## Interactive execution of nstelluric may produce better results.

# If the calibrations were reduced in a difference directory:
#imdelete xtrnS20040615S0145_comb ver- >& dev$null
#copy caldir$xtrnS20040615S0145_comb.fits . ver-

imdelete axtrnS20040615S0156_stack ver- >& dev$null
nstelluric xtrnS20040615S0156_stack xtrnS20040615S0145_comb \
    thresh=0.01 high=5 fitord=15 
splot axtrnS20040615S0156_stack[sci,1]
specplot "xtrnS20040615S0156_stack[sci,1],axtrnS20040615S0156_stack[sci,1],\
    xtrnS20040615S0145_comb[sci,1]"



######################### FINISHED  ##########################################  

# clean up
# keep only final stacks, transformed and extracted

#delete tmp*,*.lis
#imdelete nS20040615S????.fits,rnS20040615S????.fits
