# Copyright(c) 2004-2006 Association of Universities for Research in Astronony, Inc.

###############################################################################
# Gemini GNIRS data reduction script                                          #
# Reduction for: XD-32 Science Data                                           #
#                                                                             #
# This script is provided to guide the GNIRS data user through the reduction  #
# process.  It is not intended to be an executable task, or a "plug-n-play"   #
# batch job.  It does show the reduction steps using actual GNIRS data and    #
# provides explanatory comments at each step.                                 #
#                                                                             #
# At a minimum, the user needs to edit this file to match his/her dataset.    #
# It is also strongly recommended that the user read the embedded comments    #
# and understand the processing steps, as the optimum steps for a given       #
# dataset may differ.                                                         #
#                                                                             #
# Once properly edited, the script can be run by cutting and pasting the      #
# text into an IRAF session, or by typing                                     #
#   gn> task $thisscript="thisscript.cl"                                      # 
#   gn> thisscript                                                            #
# in the IRAF session.                                                        #
#                                                                             #
# WARNING: Certain INTERACTIVE steps in the process MAY NOT WORK WELL         #
#          with the 'cl < thisscriptname.cl' syntax).  Likewise, when         #
#          cutting/pasting, DO NOT paste commands directly after an           #
#          interactive task in one paste (that is paste the following         #
#          command after the interactive task).                               #
#                                                                             #
# Note that this script is designed to be re-run as needed, and so each step  #
# is preceeded by a command to delete the files created in that step.         #
###############################################################################
#
# Science Dataset:
#
# Observation UT date : 2006-may-07
# Data filename prefix: S20060507S
# File numbers:
#       Objects :       62-69      
#
# The data used in this script are not yet available in the Gemini Science Archive. 

# This reduction requires the following calibration files:
#    Flat: final_flat.fits
#    Arc:  warc_comb
#    Pinhole: pinhole
#
# The arc and pinhole must have database files in the database directory.
#
# These files can be created with the XD-Cal example.
#    

######################################################################
		### Initialize tasks ###
######################################################################
# Load required tasks/packages
gemini
gnirs

# set default parameters
# use gnirs defaults except where specified on command lines below
unlearn gemini
unlearn gemtools
unlearn gnirs



######################################################################
		### define logfile and database ###
######################################################################

# to start from scratch, delete existing log file
#delete "gnirs_xd32_sci.log" ver- >& dev$null
gnirs.logfile = "gnirs_xd32_sci.log"

# set database directory
gnirs.database = "gnirs_xd32_database"

# load default header keywords for gnirs and set display
nsheaders gnirs
set stdimage=imt1024



######################################################################
		### Create lists and path to data ###
######################################################################

#set data path
set rawdir="/net/sabrina/staging5/gemini_testdata/gnirs_testdata/xdisp/"

# If the calibration were reduced in a different directory, define 'caldir'
#set caldir=rawdir$XD32example-CAL/

delete ObjectA.list,All.list ver- >& dev$null

gemlist S20060507S 62-69 > ObjectA.list
concat ObjectA.list > All.list



######################################################################
		### nsprepare the data ###
######################################################################

# Examine all data visually first.  If data show a strong vertical striping
# pattern (due to variable bias problem on readout), run nvnoise on the raw
# data to correct this.  This task subtracts a constant from each pixel
# and so does not add any noise.
#
### If nvnoise is run, all subsequent files will have a 'v' preceeding the 
### file prefix.
#
#imdelete v@All.list ver- >& dev$null
#nvnoise @All.list
#

#- Run NSPREPARE to correct for non-linearity, determine MDF shifts,
# fix headers and create variance and data quality planes.  If a flat 
# is included in the file list and shiftx and shifty = INDEF, nsprepare 
# will determine the MDF shift from the flat and insert in all file headers.  
# Use fl_forcewcs=yes to correct the WCS keywords in the headers.
# Bad pixel mask added to GNIRS IRAF package with version 1.9.

## NOTE: It is best to nsprepare the science data with the calibration data
##       so that a flat can be used to determine the shift (shiftx/y=INDEF).  
##       If the science data is done seperately, use "shiftimage" to match the 
##       shift to the flat data.  It is important that the calibration and 
##       science data have the same shift.  In this example, the shift is taken 
##       from final_flat, the flatfield created by the XD32 CAL example.

imdelete n@All.list ver- >& dev$null
nsprepare @All.list rawpath=rawdir$ shiftimage="final_flat.fits" fl_force+ \
    bpm="gnirs$data/gnirs_2005sep24_bpm.fits"

#If the calibrations were reduced in a different directory:
#nsprepare @All.list rawpath=rawdir$ shiftimage="caldir$final_flat.fits" \
#   fl_force+ bpm="gnirs$data/gnirs_2005sep24_bpm.fits"


######################################################################
		### Reduce Object ###
######################################################################

## There are 3 ways to determine the correct frames for sky subtraction
## in nsreduce.  If the frames are evenly spaced in time (i.e., ABBA
## dither pattern), the easiest is "skyrange=INDEF" (the default), which
## will determine the best sky frame for each image from the input list.
## One can also set the skyrange (in seconds) by hand, or provide a list
## of sky images directly.  See nsreduce help for more information.
## If two or more images meet the sky selection criteria (skyrange and
## nodsize), they will be combined to create the sky frame that is subtracted.


# The following lines can be used to determine the spacing in time between
# the object frames.
#delete tmp.lis ver- >& dev$null
#gemextn n@ObjectA.list proc=expand index=0 > tmp.lis
#
#hselect @tmp.lis $I,ut yes | tee (gnirs.logfile, append+)
#
# Frames are x secs apart

### - Reduce Objects ###
## object A ##

imdelete rn@ObjectA.list ver- >& dev$null
nsreduce n@ObjectA.list fl_corner+ fl_process+ fl_nsappwave- fl_sky+ \
    skyrange=INDEF fl_flat+ flatimage="final_flat.fits" nodsize=3.0

# If the calibrations were reduced in a different directory:
#nsreduce n@ObjectA.list fl_corner+ fl_process+ fl_nsappwave- fl_sky+ \
#    skyrange=INDEF fl_flat+ flatimage="caldir$final_flat.fits" nodsize=3.0


### - Combine Objects ###
# Using all generic names here for ease of re-using script; default
# output names will append "_comb" to the first name in the input list.

## object A ##
imdelete objA_comb.fits ver- >& dev$null
nscombine rn@ObjectA.list  output="objA_comb"
nxdisplay objA_comb 1

### - nstransform & nsextract ###

## NSTRANSFORM is used to apply the wavelength solution and an
## sdistortion correction, if desired.  It can be run with either
## lamp or sdist or both.  It is best run interactively.
## If only the lamp solution is applied, the spectra will not be
## "straightened", but they will be flipped in y because of the wavelength
## solution.  To also straighten, add the pinhole (sdisttrans="pinhole").
                                                                                
## The interactive mode of nstransform puts you into fitcoords, which
## plots the fit and the residuals for the 2-dimensional xy fit for each
## transformation (lamp and sdist).  See help pages for nstransform, fitcoords
## and transform for more information.  For the lamp solution, xorder=1
## and yorder=3 works best, and outlying points should be deleted. For the
## sdist (pinhole) solution, use xorder=4 and yorder=4 for XD32 data (lower
## orders, 2-3, for XD111 data); usually the first y point and the last few
## y points in each order should be deleted.  (Hit "f" to redo the fit.)  If 
## the pinhole trace is lost, which may happen for the higher orders (especially 
## pre-July 2005 data), more upper y datapoints may need to be deleted.
                                                                                
## NOTE that it is important to apply the same solution for the telluric and
## the science data.

## Object A ##
##Best run interactively - paste onto command line

# If the calibrations were reduced in a different directory:
#imdelete warc_comb ver- >& dev$null
#copy caldir$warc_comb.fits .
#if (access ("gnirs_xd32_database"))
#    delete gnirs_xd32_database/* ver- >& dev$null
#else
#    !mkdir gnirs_xd32_database
#copy caldir$gnirs_xd32_database/* gnirs_xd32_database/
#copy caldir$pinhole.fits .

imdelete tobjA_comb.fits ver- >& dev$null
nstransform objA_comb.fits lamptrans="warc_comb.fits" sdisttrans="pinhole" \
    fl_inter+ xorder=4 yorder=4

nxdisplay tobjA_comb.fits  1

imdelete xtobjA_comb.fits ver- >& dev$null
nsextract tobjA_comb.fits line=750 nsum=20 upper=6 low=-6 fl_inter+ \
    fl_trace+ tr_nsum=5 tr_step=2

# For strong signal, nsextract can be run non-interactively with default 
# parameters.
#nsextract tobjA_comb fl_trace+

# Data taken before July 2005 are littered with radiation events. For these
# data cosmic ray rejection is needed earlier in the reduction process.
# Data taken after July 2005 will look much better as they do not suffer from 
# excessive radiation events.

######################### FINISHED  ##########################################  



#- delete extra files
#- keep those lines commented until you are done

#delete *lis
#delete tmp*
#imdelete nS20060507S????.fits ver- >& dev$null
#imdelete rnS20060507S????.fits ver- >& dev$null

  
