# Cpyright(c) 2006 Association of Universities for Research in Astronomy, Inc.
#
# NFTRANSFORM -- Transform long slit images to rectified and wavelength
# calibrated format.  This requires the 2D dispersion and distortion functions
# to be previously fit by NFFITCOORDS.

procedure nftransform (inimages) 

# Rectify NIFS spectra using available wavelength calibrations
# and S-distoritions.
#
# Version: Sep 20, 2002 IJ,MT,JJ v1.4 release
#          Aug 19, 2003 KL  IRAF2.12 - new parameter, addonly, in hedit
#          Oct 29, 2003 KL  moved from niri to gnirs package
#          Aug  8, 2005 FV  separated fitcoords and transform to separate tasks

char    inimages   {prompt = "Input GNIRS/NIRI spectra"}                                     # OLDP-1-primary-single-prefix=t
char    outspectra {"", prompt = "Output spectra"}                                           # OLDP-1-output
char    outprefix  {"t", prompt = "Prefix for output spectra"}                               # OLDP-4
int     dispaxis   {1,min=1,max=2,prompt="Dispersion axis if not defined in the header"}     # OLDP-3
char    interptype {"poly3",enum="nearest|linear|poly3|poly5|spline3",prompt = "Interpolation type for transform"}                 # OLDP-2
bool    xlog       {no, prompt = "Logarithmic x coordinate for transform"}                   # OLDP-2
bool    ylog       {no, prompt = "Logarithmic y coordinate for transform"}                   # OLDP-2
real    blank      {0., prompt = "Value for out of bounds pixels"}                           #
char    logfile    {"", prompt = "Logfile"}                                                  # OLDP-1
bool    verbose    {yes, prompt = "Verbose"}                                                 # OLDP-2
int     status     {0, prompt = "Exit status (0=Good)"}                                      # OLDP-4
struct* scanfile1  {"", prompt = "Internal use"}                                             # OLDP-4

begin
        char    l_inimages = ""
        char    l_outspectra = ""
        char    l_outprefix = ""
        char    l_interptype = ""
        char    l_logfile = ""
        char    l_key_dispaxis = ""
        char    l_sci_ext = ""
        char    l_var_ext = ""
        char    l_dq_ext = ""

        int     l_dispaxis
        int     nlamps, nsdist, nver, version, nx, ny, nxx, nyy
        int     nbad, nfiles, junk, nfail, dqscale, iextn

        bool    l_xlog, l_ylog, l_verbose, intdbg

        char    badhdr, keyfound
        char    l_database, l_fitname, l_fitname_lamp, l_fitname_sdist
        char    img, imgin, imgout, sec, phu, key, line
        char    extn[3], outimg, tmpin, tmpdqint, tmpdqint2
        real    dx, dy, x1, x2, y1, y2
        struct  l_date, sline
        
        # Define temporary files
        tmpin = mktemp ("tmpin") 

        # Cache parameter files.
        cache ("gemextn", "nfcheck", "gemdate")
        
        # Initialize
        status = 1
        intdbg = no
        dqscale = 1000

        junk = fscan (inimages, l_inimages)
        junk = fscan (outspectra, l_outspectra)
        junk = fscan (outprefix, l_outprefix)
        junk = fscan (interptype, l_interptype)
        junk = fscan (logfile, l_logfile)

        l_dispaxis = dispaxis
        l_xlog     = xlog
        l_ylog     = ylog
        l_verbose  = verbose

        badhdr = ""
        junk = fscan (  nsheaders.key_dispaxis, l_key_dispaxis)
        if ("" == l_key_dispaxis) badhdr = badhdr + " key_dispaxis"
        junk = fscan (  nsheaders.sci_ext, l_sci_ext)
        if ("" == l_sci_ext) badhdr = badhdr + " sci_ext"
        junk = fscan (  nsheaders.var_ext, l_var_ext)
        if ("" == l_var_ext) badhdr = badhdr + " var_ext"
        junk = fscan (  nsheaders.dq_ext, l_dq_ext)
        if ("" == l_dq_ext) badhdr = badhdr + " dq_ext"

        extn[1] = l_sci_ext
        extn[2] = l_var_ext
        extn[3] = l_dq_ext

        if (l_logfile == "") {
            junk = fscan (nifs.logfile, l_logfile)
            if (l_logfile == "") {
                l_logfile = "nifs.log"
                printlog ("WARNING - NFTRANSFORM:  Both nftransform.logfile \
                    and nifs.logfile", l_logfile, verbose+) 
                printlog ("                        are empty.  \
                    Using " // l_logfile, l_logfile, verbose+) 
            }
        }

        printlog ("-------------------------------------------------------------------------------", l_logfile, l_verbose) 
        date | scan (l_date) 
        printlog ("NFTRANSFORM -- " // l_date, l_logfile, l_verbose) 
        printlog ("", l_logfile, l_verbose) 

        if ("" != badhdr) {
            printlog ("ERROR - NFTRANSFORM: Parameter(s) missing from \
                nsheaders: " // badhdr, l_logfile, verbose+) 
            goto clean
        }
 
        # Expand and verify input.
        nfcheck (input=l_inimages, output=l_outspectra, prefix=l_outprefix,
            outlist=tmpin, name="NFTRANSFORM", dependflag="NFFITCOO",
            procflag="NFTRANSF", sci_ext=l_sci_ext, logfile=l_logfile,
            verbose=l_verbose, vverbose=intdbg)
        if (nfcheck.status == 1)
            goto clean
        nfiles = nfcheck.nver
        nver = nfcheck.nver

        # Main loop for transformation
        scanfile1 = tmpin
        while (fscan (scanfile1, imgin, imgout) != EOF) {
            if (intdbg) print ("processing " // imgin)

            # Initialize.
            nlamps = 0; nsdist = 0; nfail = 0
            l_dispaxis = dispaxis

            phu = imgin // "[0]"
            hselect (phu, "FCNLAMPS", yes) | scan (nlamps)
            hselect (phu, "FCNSDIST", yes) | scan (nsdist)
            hselect (phu, l_key_dispaxis, yes) | scan (l_dispaxis)
            x1 = INDEF; hselect (phu, "FCX1", yes) | scan (x1)
            x2 = INDEF; hselect (phu, "FCX2", yes) | scan (x2)
            dx = INDEF; hselect (phu, "FCDX", yes) | scan (dx)
            nx = INDEF; hselect (phu, "FCNX", yes) | scan (nx)
            y1 = INDEF; hselect (phu, "FCY1", yes) | scan (y1)
            y2 = INDEF; hselect (phu, "FCY2", yes) | scan (y2)
            dy = INDEF; hselect (phu, "FCDY", yes) | scan (dy)
            ny = INDEF; hselect (phu, "FCNY", yes) | scan (ny)

            for (version = 1; version <= nver; version = version + 1) {

                if (intdbg) print ("version " // version)
                img = imgin // "[" // l_sci_ext // "," // version // "]"

                # Copy the PHU the first time.
                if (version == 1) {
                    if (intdbg) print ("copying phu")
                    imcopy (imgin // "[0]", imgout, verbose-) 
                }

                if (intdbg) print ("get fitcoords information") 

                l_database = ""
                hselect (img, "FCDB", yes) | scan (l_database)
                if (l_database == "") {
                    printlog ("ERROR - NFTRANSFORM: No wavelength or S-distortion \
                        transformations defined", l_logfile, verbose+) 
                    goto clean
                }
                l_fitname_lamp = ""; l_fitname_sdist = ""
                hselect (img, "FCFIT1", yes) | scan (l_fitname_lamp)
                hselect (img, "FCFIT2", yes) | scan (l_fitname_sdist)
                l_fitname = l_fitname_lamp // "," // l_fitname_sdist
                #x1 = INDEF; hselect (img, "FCX1", yes) | scan (x1)
                #x2 = INDEF; hselect (img, "FCX2", yes) | scan (x2)
                #dx = INDEF; hselect (img, "FCDX", yes) | scan (dx)
                #nx = INDEF; hselect (img, "FCNX", yes) | scan (nx)
                #y1 = INDEF; hselect (img, "FCY1", yes) | scan (y1)
                #y2 = INDEF; hselect (img, "FCY2", yes) | scan (y2)
                #dy = INDEF; hselect (img, "FCDY", yes) | scan (dy)
                #ny = INDEF; hselect (img, "FCNY", yes) | scan (ny)

                printlog ("NFTRANSFORM: Fitnames for " // imgin \
                    // ": " // l_fitname, l_logfile, l_verbose) 

                if (l_fitname_lamp != "" &&
                    access (l_database//"/fc"//l_fitname_lamp) == no) {
                    printlog ("ERROR - NFTRANSFORM: Wavelength \
                        transformations not found", l_logfile, verbose+) 
                    goto clean
                }
                if (l_fitname_sdist != "" &&
                    access (l_database//"/fc"//l_fitname_sdist) == no) {
                    printlog ("ERROR - NFTRANSFORM: S-distortion \
                        transformations not found", l_logfile, verbose+) 
                    goto clean
                }

                if (intdbg) {
                    print (l_database, ", ", l_fitname)
                    print (x1, x2, dx, nx)
                    print (y1, y2, dy, ny)
                }

                # Work around for WCS problem.
                junk = 2 
		    hselect (img, "WCSDIM", yes) | scan (junk)
                if (junk != 2)
                    hedit (img, "WCSDIM", add-, addo-, del+, ver-, show-, update+)

                # Transform the data using the fit information
                if (intdbg) print ("transform sci") 
                outimg = imgout // "[" // l_sci_ext // "," // \
                    version // ",append]"
                transform (img, outimg, fitnames = l_fitname,
                    database = l_database, interptype = l_interptype,
                    xlog = l_xlog, ylog = l_ylog, flux = yes,
                    logfiles = l_logfile, dx=dx, dy=dy,
                    x1=x1, x2=x2, y1=y1, y2=y2, nx=nx, ny=ny, blank=blank) 
                hedit (outimg, "FC*", add-, addo-, del+, ver-, show-, update+)

                if (junk != 2)
                    hedit (img, "WCSDIM", junk, add+, ver-, show-, update+)

                # Convert WCS to 3D for use with NIFCUBE.
                nfwcs (outimg, mdf="!mdf_file", xmdfname="DEC", ymdfname="RA",
                    dymdfname="slitsize_x", pixscale="!pixscale",
                    xoffset="0.", yoffset="0.", keepwcs+)

                img = imgin // "[" // l_var_ext // "," // version // "]"
                if (imaccess (img)) {
                    if (intdbg) print ("transform var") 
                    transform (img, imgout // "[" // l_var_ext // "," \
                        // version // ",append]", fitnames = l_fitname,
                        database = l_database, interptype = l_interptype,
                        xlog = l_xlog, ylog = l_ylog, flux = yes,
                        logfiles = l_logfile, dx=dx, dy=dy,
                        x1=x1, x2=x2, y1=y1, y2=y2, nx=nx, ny=ny,
                        blank=0) 
                }

                img = imgin // "[" // l_dq_ext // "," // version // "]"
                if (imaccess (img)) {
                    if (intdbg) print ("transform dq") 
                    tmpdqint = mktemp ("tmpdqintimg") 

                    # need to scale dq because otherwise interpolation
                    # is in integers (or convert to real)
                    # and back again!
                    imarith (img, "*", dqscale, tmpdqint, title="",
                        divzero=0, hparams="", pixtype="",
                        calctype="", verbose-, noact-)

                    # transform (must be linear to avoid bleeding)
                    hedit (tmpdqint, "dispaxis", dispaxis, add+,
                        verify-, show-,  update+)
                    transform (tmpdqint, tmpdqint, fitnames = l_fitname,
                        database = l_database, interptype = "linear",
                        xlog = l_xlog, ylog = l_ylog, flux = yes,
                        logfiles = l_logfile, dx=dx, dy=dy,
                        x1=x1, x2=x2, y1=y1, y2=y2, nx=nx, ny=ny,
                        blank=1) 

                    # anything non-zero has been contaminated
                    # so set to 1 (all we can do, really)
                    imexpr ("a > 0 ? 1 : 0",
                        imgout // "[" // l_dq_ext // "," \
                        // version // ",append]", tmpdqint,
                        dims = "auto", intype = "auto", outtype = "int",
                        refim = "auto", bwidth = 0, btype = "nearest",
                        bpixval = 0, rangecheck+, verbose-,
                        exprdb = "none")

                    imdelete (tmpdqint, verify-) 
                }

                # Copy across nsappwave info if no wavelength transform
                if (l_fitname_lamp == "") {
                    for (iextn = 1; iextn <= 3; iextn = iextn+1) {
                        sec = "[" // extn[iext] // "," // version // "]"
                        if (!(imaccess(imgin//sec) && imaccess(imgout//sec)))
                            next

                        if (intdbg) print ("nsappwave info for " // sec)
                        if (intdbg) print ("from " // imgin)
                        if (intdbg) print ("to " // imgout)

                        key = "CTYPE" // l_dispaxis
                        hselect (imgin // sec, key, yes) | scan (sline)
                        if (fscan (sline, line) > 0) {
                            hedit (imgout // sec, key,
                                value=sline, add+, addonly-,
                                delete-, verify-, show-, update+)
                        }

                        key = "CRPIX" // l_dispaxis
                        hselect (imgin // sec, key, yes) | scan (sline)
                        if (fscan (sline, line) > 0) {
                            hedit (imgout // sec, key,
                                value=sline, add+, addonly-,
                                delete-, verify-, show-, update+)
                        }

                        key = "CRVAL" // l_dispaxis
                        hselect (imgin // sec, key, yes) | scan (sline)
                        if (fscan (sline, line) > 0) {
                            hedit (imgout // sec, key,
                                value=sline, add+, addonly-,
                                delete-, verify-, show-, update+)
                        }

                        key = "CD" // l_dispaxis // "_" // l_dispaxis
                        hselect (imgin // sec, key, yes) | scan (sline)
                        if (fscan (sline, line) > 0) {
                            hedit (imgout // sec, key,
                                value=sline, add+, addonly-,
                                delete-, verify-, show-, update+)
                        }

                        key = "WAT" // l_dispaxis // "_000"
                        hselect (imgin // sec, key, yes) | scan (sline)
                        if (fscan (sline, line) > 0) {
                            hedit (imgout // sec, key,
                                value=sline, add+, addonly-,
                                delete-, verify-, show-, update+)
                        }

                        key = "WAT" // l_dispaxis // "_001"
                        hselect (imgin // sec, key, yes) | scan (sline)
                        if (fscan (sline, line) > 0) {
                            hedit (imgout // sec, key,
                                value=sline, add+, addonly-,
                                delete-, verify-, show-, update+)
                        }

                        key = "WAT" // l_dispaxis // "_002"
                        hselect (imgin // sec, key, yes) | scan (sline)
                        if (fscan (sline, line) > 0) {
                            hedit (imgout // sec, key,
                                value=sline, add+, addonly-,
                                delete-, verify-, show-, update+)
                        }
                    }
                }
            }

            # Update the header
            if (nfail > 0) {
                hselect (imgout // "[0]", "NSCIEXT", yes) | scan (junk)
                hedit (imgout // "[0]", "NSCIEXT",
                    (junk - nfail), add+, addonly-, delete-,
                    verify-, show-, update+)
            }

            # Insert the MDF
            if (intdbg) print ("and copy MDF")
            gemextn (imgin, check="exists,table", process="expand",
                index="1-", extname="MDF", extversion="", ikparams="",
                omit="name,version", replace="", outfile="STDOUT",
                logfile=l_logfile, glogpars="", verbose-) | scan (sline)
            if (gemextn.count == 0 || gemextn.fail_count != 0) {
                printlog ("WARNING - NFTRANSFORM: No MDF in " \
                    // imgin, l_logfile, verbose+)
            } else {
                fxinsert (sline, imgout // ".fits[0]", groups="",
                    verbose=intdbg)
            }
        
            gemdate ()
            nhedit (imgout//"[0]", "NFTRANSF", gemdate.outdate,
                "UT Time stamp for NFTRANSFORM", comfile="NULL", after="",
                before="", update+, add+, addonly-, delete-, verify-, show-)
            nhedit (imgout//"[0]", "GEM-TLM", gemdate.outdate,
                "UT Last modification with GEMINI", comfile="NULL", after="",
                before="", update+, add+, addonly-, delete-, verify-, show-)
        } #Closing the 'while' loop

        # Completed successfully
        status = 0

clean:
        if (status == 0) {
            printlog ("", l_logfile, no) 
            printlog ("NFTRANSFORM  Exit status good", l_logfile, l_verbose) 
            printlog ("-------------------------------------------------------------\
                ------------------", l_logfile, l_verbose) 
        }
        scanfile1 = ""
        delete (tmpin, ver-, >& "dev$null") 
end
